@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 import argparse
 import client
+import json
 
 def main():
     parser = argparse.ArgumentParser()
@@ -9,7 +10,7 @@ def main():
     args = parser.parse_args()
 
     conn = client.Client((args.host, args.port))
-    print(conn.call("server.version", "health_check", "1.4")["result"])
+    print(json.dumps(conn.call([client.request("server.version", "health_check", "1.4")])))
 
 if __name__ == '__main__':
 	main()
